<?php
/**
 * RdKafkaFactory.php
 * rdkafka connection factory.
 * @version    170410:1
 * @author     karminski <code.karminski@outlook.com>
 *
 * @changelogs
 *             170410:1 INIT version.
 */

namespace SaltFish;

/**
 * class RdKafkaFactory
 */
class RdKafkaFactory
{

    const ROLE_PRODUCER = 'producer';
    const ROLE_CONSUMER = 'consumer';

    private static $instance;
    public $h;
    public $pool = array();


    /**
     * [__construct description]
     * @param array &$h , redis all config
     */
    private function __construct()
    {
        $this->h = Config::get('queue');
        return true;
    }

    /**
     * [getInstance description]
     * @return [type] [description]
     */
    public static function getInstance()
    {
        if (empty(self::$instance)) {
            self::$instance = new RdKafkaFactory;
        }
        return self::$instance;
    }

    /**
     * [spawn description]
     * @param  [type] $role      [description]
     * @param  [type] $queueName [description]
     * @param  string $groupName [description]
     * @return [type]            [description]
     */
    public static function &spawn($role, $queueName, $groupName = "")
    {
        $instance = self::getInstance();
        // if alerady spawn return pool result
        $index = "{$queueName}.{$role}";
        if (!empty($instance->pool[$index])) return $instance->pool[$index];
        // spawn
        if ($role === self::ROLE_PRODUCER) {
            $RdKafkaInstance = new \RdKafka\Producer();
            $RdKafkaInstance->setLogLevel(LOG_DEBUG);
            $RdKafkaInstance->addBrokers($instance->h[$queueName]['brokers']);
            $RdKafkaProducerProxy = new RdKafkaProducerProxy;
            $RdKafkaProducerProxy->setInstance($RdKafkaInstance);
            $instance->pool[$index] = $RdKafkaProducerProxy;
        } elseif ($role === self::ROLE_CONSUMER) {
            $conf = new \RdKafka\Conf();
            $conf->setRebalanceCb(function (\RdKafka\KafkaConsumer $kafka, $err, array $partitions = null) {
                switch ($err) {
                    case RD_KAFKA_RESP_ERR__ASSIGN_PARTITIONS:
                        $kafka->assign($partitions);
                        break;

                    case RD_KAFKA_RESP_ERR__REVOKE_PARTITIONS:
                        $kafka->assign(NULL);
                        break;

                    default:
                        throw new \Exception($err);
                }
            });
            $conf->set('group.id', $groupName);
            $conf->set('metadata.broker.list', $instance->h[$queueName]['brokers']);
            $topicConf = new \RdKafka\TopicConf();
            $topicConf->set('auto.offset.reset', 'smallest');
            $conf->setDefaultTopicConf($topicConf);
            $RdKafkaInstance = new \RdKafka\KafkaConsumer($conf);
            $RdKafkaConsumerProxy = new RdKafkaConsumerProxy;
            $RdKafkaConsumerProxy->setInstance($RdKafkaInstance);
            $instance->pool[$index] = $RdKafkaConsumerProxy;
        } else {
            throw new Exception("Unexpected role.", 1);
        }
        return $instance->pool[$index];
    }


}